@@ -32,7 +32,7 @@ from glance_store import capabilities
 from glance_store.common import utils
 import glance_store.driver
 from glance_store import exceptions
-from glance_store.i18n import _, _LE, _LW, _LI
+from glance_store.i18n import _, _LE, _LI
 import glance_store.location
 
 try:
@@ -53,7 +53,7 @@ LOG = logging.getLogger(__name__)
 _CINDER_OPTS = [
     cfg.StrOpt('cinder_catalog_info',
                default='volumev2::publicURL',
-               help=_("""
+               help="""
 Information to match when looking for cinder in the service catalog.
 
 When the ``cinder_endpoint_template`` is not set and any of
@@ -79,10 +79,10 @@ Related options:
     * cinder_store_project_name
     * cinder_store_password
 
-""")),
+"""),
     cfg.StrOpt('cinder_endpoint_template',
                default=None,
-               help=_("""
+               help="""
 Override service catalog lookup with template for cinder endpoint.
 
 When this option is set, this value is used to generate cinder endpoint,
@@ -105,10 +105,10 @@ Related options:
     * cinder_store_password
     * cinder_catalog_info
 
-""")),
+"""),
     cfg.StrOpt('cinder_os_region_name', deprecated_name='os_region_name',
                default=None,
-               help=_("""
+               help="""
 Region name to lookup cinder service from the service catalog.
 
 This is used only when ``cinder_catalog_info`` is used for determining the
@@ -122,9 +122,9 @@ Possible values:
 Related options:
     * cinder_catalog_info
 
-""")),
+"""),
     cfg.StrOpt('cinder_ca_certificates_file',
-               help=_("""
+               help="""
 Location of a CA certificates file used for cinder client requests.
 
 The specified CA certificates file, if set, is used to verify cinder
@@ -137,11 +137,11 @@ Possible values:
 Related options:
     * cinder_api_insecure
 
-""")),
+"""),
     cfg.IntOpt('cinder_http_retries',
                min=0,
                default=3,
-               help=_("""
+               help="""
 Number of cinderclient retries on failed http calls.
 
 When a call failed by any errors, cinderclient will retry the call up to the
@@ -153,11 +153,11 @@ Possible values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.IntOpt('cinder_state_transition_timeout',
                min=0,
                default=300,
-               help=_("""
+               help="""
 Time period, in seconds, to wait for a cinder volume transition to
 complete.
 
@@ -174,10 +174,10 @@ Possible values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.BoolOpt('cinder_api_insecure',
                 default=False,
-                help=_("""
+                help="""
 Allow to perform insecure SSL requests to cinder.
 
 If this option is set to True, HTTPS endpoint connection is verified using the
@@ -190,10 +190,10 @@ Possible values:
 Related options:
     * cinder_ca_certificates_file
 
-""")),
+"""),
     cfg.StrOpt('cinder_store_auth_address',
                default=None,
-               help=_("""
+               help="""
 The address where the cinder authentication service is listening.
 
 When all of ``cinder_store_auth_address``, ``cinder_store_user_name``,
@@ -215,10 +215,10 @@ Related options:
     * cinder_store_password
     * cinder_store_project_name
 
-""")),
+"""),
     cfg.StrOpt('cinder_store_user_name',
                default=None,
-               help=_("""
+               help="""
 User name to authenticate against cinder.
 
 This must be used with all the following related options. If any of these are
@@ -232,9 +232,9 @@ Related options:
     * cinder_store_password
     * cinder_store_project_name
 
-""")),
+"""),
     cfg.StrOpt('cinder_store_password', secret=True,
-               help=_("""
+               help="""
 Password for the user authenticating against cinder.
 
 This must be used with all the following related options. If any of these are
@@ -248,10 +248,10 @@ Related options:
     * cinder_store_user_name
     * cinder_store_project_name
 
-""")),
+"""),
     cfg.StrOpt('cinder_store_project_name',
                default=None,
-               help=_("""
+               help="""
 Project name where the image volume is stored in cinder.
 
 If this configuration option is not set, the project in current context is
@@ -268,10 +268,10 @@ Related options:
     * ``cinder_store_user_name``
     * ``cinder_store_password``
 
-""")),
+"""),
     cfg.StrOpt('rootwrap_config',
                default='/etc/glance/rootwrap.conf',
-               help=_("""
+               help="""
 Path to the rootwrap configuration file to use for running commands as root.
 
 The cinder store requires root privileges to operate the image volumes (for
@@ -285,10 +285,10 @@ Possible values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.StrOpt('cinder_volume_type',
                default=None,
-               help=_("""
+               help="""
 Volume type that will be used for volume creation in cinder.
 
 Some cinder backends can have several volume types to optimize storage usage.
@@ -304,7 +304,7 @@ Possible values:
 Related options:
     * None
 
-""")),
+"""),
 ]
 
 
@@ -435,13 +435,15 @@ class Store(glance_store.driver.Store):
 
     def __init__(self, *args, **kargs):
         super(Store, self).__init__(*args, **kargs)
-        LOG.warning(_LW("Cinder store is considered experimental. "
-                        "Current deployers should be aware that the use "
-                        "of it in production right now may be risky."))
+        if self.backend_group:
+            self._set_url_prefix()
 
     def get_schemes(self):
         return ('cinder',)
 
+    def _set_url_prefix(self):
+        self._url_prefix = "cinder://"
+
     def _check_context(self, context, require_tenant=False):
         user_overriden = is_user_overriden(self.conf,
                                            backend=self.backend_group)
@@ -645,8 +647,9 @@ class Store(glance_store.driver.Store):
                               "internal error."))
             return 0
 
+    @glance_store.driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """
         Stores an image file with supplied identifier to the backend
@@ -656,19 +659,21 @@ class Store(glance_store.driver.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
         :param context: The request context
         :param verifier: An object used to verify signatures for images
 
-        :returns: tuple of URL in backing store, bytes written, checksum
-                and a dictionary with storage system specific information
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
         :raises: `glance_store.exceptions.Duplicate` if the image already
-                existed
+                 exists
         """
 
         self._check_context(context, require_tenant=True)
         client = get_cinderclient(self.conf, context,
                                   backend=self.backend_group)
-
+        os_hash_value = hashlib.new(str(hashing_algo))
         checksum = hashlib.md5()
         bytes_written = 0
         size_gb = int(math.ceil(float(image_size) / units.Gi))
@@ -712,6 +717,7 @@ class Store(glance_store.driver.Store):
                         if not buf:
                             need_extend = False
                             break
+                        os_hash_value.update(buf)
                         checksum.update(buf)
                         if verifier:
                             verifier.update(buf)
@@ -757,6 +763,7 @@ class Store(glance_store.driver.Store):
             volume.update_all_metadata(metadata)
         volume.update_readonly_flag(volume, True)
 
+        hash_hex = os_hash_value.hexdigest()
         checksum_hex = checksum.hexdigest()
 
         LOG.debug("Wrote %(bytes_written)d bytes to volume %(volume_id)s "
@@ -769,8 +776,11 @@ class Store(glance_store.driver.Store):
         if self.backend_group:
             image_metadata['backend'] = u"%s" % self.backend_group
 
-        return ('cinder://%s' % volume.id, bytes_written,
-                checksum_hex, image_metadata)
+        return ('cinder://%s' % volume.id,
+                bytes_written,
+                checksum_hex,
+                hash_hex,
+                image_metadata)
 
     @capabilities.check
     def delete(self, location, context=None):
