@@ -46,7 +46,7 @@ LOG = logging.getLogger(__name__)
 _FILESYSTEM_CONFIGS = [
     cfg.StrOpt('filesystem_store_datadir',
                default='/var/lib/glance/images',
-               help=_("""
+               help="""
 Directory to which the filesystem backend store writes images.
 
 Upon start up, Glance creates the directory if it doesn't already
@@ -69,9 +69,9 @@ Related options:
     * ``filesystem_store_datadirs``
     * ``filesystem_store_file_perm``
 
-""")),
+"""),
     cfg.MultiStrOpt('filesystem_store_datadirs',
-                    help=_("""
+                    help="""
 List of directories and their priorities to which the filesystem
 backend store writes images.
 
@@ -105,9 +105,9 @@ Related options:
     * ``filesystem_store_datadir``
     * ``filesystem_store_file_perm``
 
-""")),
+"""),
     cfg.StrOpt('filesystem_store_metadata_file',
-               help=_("""
+               help="""
 Filesystem store metadata file.
 
 The path to a file which contains the metadata to be returned with
@@ -122,10 +122,10 @@ Possible values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.IntOpt('filesystem_store_file_perm',
                default=0,
-               help=_("""
+               help="""
 File access permissions for the image files.
 
 Set the intended file access permissions for image data. This provides
@@ -148,7 +148,24 @@ Possible values:
 Related options:
     * None
 
-"""))]
+"""),
+    cfg.IntOpt('filesystem_store_chunk_size',
+               default=64 * units.Ki,
+               min=1,
+               help="""
+Chunk size, in bytes.
+
+The chunk size used when reading or writing image files. Raising this value
+may improve the throughput but it may also slightly increase the memory usage
+when handling a large number of requests.
+
+Possible Values:
+    * Any positive integer value
+
+Related options:
+    * None
+
+""")]
 
 MULTI_FILESYSTEM_METADATA_SCHEMA = {
     "type": "array",
@@ -243,8 +260,6 @@ class Store(glance_store.driver.Store):
                      capabilities.BitMasks.WRITE_ACCESS |
                      capabilities.BitMasks.DRIVER_REUSABLE)
     OPTIONS = _FILESYSTEM_CONFIGS
-    READ_CHUNKSIZE = 64 * units.Ki
-    WRITE_CHUNKSIZE = READ_CHUNKSIZE
     FILESYSTEM_STORE_METADATA = None
 
     def get_schemes(self):
@@ -384,19 +399,18 @@ class Store(glance_store.driver.Store):
         itself, it should raise `exceptions.BadStoreConfiguration`
         """
         if self.backend_group:
-            fdir = getattr(
-                self.conf, self.backend_group).filesystem_store_datadir
-            fdirs = getattr(
-                self.conf, self.backend_group).filesystem_store_datadirs
-            fstore_perm = getattr(
-                self.conf, self.backend_group).filesystem_store_file_perm
-            meta_file = getattr(
-                self.conf, self.backend_group).filesystem_store_metadata_file
+            store_conf = getattr(self.conf, self.backend_group)
         else:
-            fdir = self.conf.glance_store.filesystem_store_datadir
-            fdirs = self.conf.glance_store.filesystem_store_datadirs
-            fstore_perm = self.conf.glance_store.filesystem_store_file_perm
-            meta_file = self.conf.glance_store.filesystem_store_metadata_file
+            store_conf = self.conf.glance_store
+
+        fdir = store_conf.filesystem_store_datadir
+        fdirs = store_conf.filesystem_store_datadirs
+        fstore_perm = store_conf.filesystem_store_file_perm
+        meta_file = store_conf.filesystem_store_metadata_file
+
+        self.chunk_size = store_conf.filesystem_store_chunk_size
+        self.READ_CHUNKSIZE = self.chunk_size
+        self.WRITE_CHUNKSIZE = self.READ_CHUNKSIZE
 
         if not (fdir or fdirs):
             reason = (_("Specify at least 'filesystem_store_datadir' or "
@@ -436,7 +450,7 @@ class Store(glance_store.driver.Store):
                 (datadir_path,
                  priority) = self._get_datadir_path_and_priority(datadir)
                 priority_paths = self.priority_data_map.setdefault(
-                    int(priority), [])
+                    priority, [])
                 self._check_directory_paths(datadir_path, directory_paths,
                                             priority_paths)
                 directory_paths.add(datadir_path)
@@ -446,10 +460,16 @@ class Store(glance_store.driver.Store):
                                         reverse=True)
 
         self._create_image_directories(directory_paths)
+        if self.backend_group:
+            self._set_url_prefix()
 
         if meta_file:
             self._validate_metadata(meta_file)
 
+    def _set_url_prefix(self):
+        path = self._find_best_datadir(0)
+        self._url_prefix = "%s://%s" % ('file', path)
+
     def _check_directory_paths(self, datadir_path, directory_paths,
                                priority_paths):
         """
@@ -490,8 +510,9 @@ class Store(glance_store.driver.Store):
         parts = [part.strip() for part in datadir.rsplit(":", 1)]
         datadir_path = parts[0]
         if len(parts) == 2 and parts[1]:
-            priority = parts[1]
-            if not priority.isdigit():
+            try:
+                priority = int(parts[1])
+            except ValueError:
                 msg = (_("Invalid priority value %(priority)s in "
                          "filesystem configuration") % {'priority': priority})
                 LOG.exception(msg)
@@ -659,8 +680,9 @@ class Store(glance_store.driver.Store):
 
         return best_datadir
 
+    @glance_store.driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """
         Stores an image file with supplied identifier to the backend
@@ -670,12 +692,15 @@ class Store(glance_store.driver.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
+        :param context: The request context
         :param verifier: An object used to verify signatures for images
 
-        :returns: tuple of URL in backing store, bytes written, checksum
-                and a dictionary with storage system specific information
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
         :raises: `glance_store.exceptions.Duplicate` if the image already
-                existed
+                 exists
 
         :note:: By default, the backend writes the image data to a file
               `/<DATADIR>/<ID>`, where <DATADIR> is the value of
@@ -688,7 +713,7 @@ class Store(glance_store.driver.Store):
 
         if os.path.exists(filepath):
             raise exceptions.Duplicate(image=filepath)
-
+        os_hash_value = hashlib.new(str(hashing_algo))
         checksum = hashlib.md5()
         bytes_written = 0
         try:
@@ -696,6 +721,7 @@ class Store(glance_store.driver.Store):
                 for buf in utils.chunkreadable(image_file,
                                                self.WRITE_CHUNKSIZE):
                     bytes_written += len(buf)
+                    os_hash_value.update(buf)
                     checksum.update(buf)
                     if verifier:
                         verifier.update(buf)
@@ -711,14 +737,16 @@ class Store(glance_store.driver.Store):
             with excutils.save_and_reraise_exception():
                 self._delete_partial(filepath, image_id)
 
+        hash_hex = os_hash_value.hexdigest()
         checksum_hex = checksum.hexdigest()
         metadata = self._get_metadata(filepath)
 
-        LOG.debug(_("Wrote %(bytes_written)d bytes to %(filepath)s with "
-                    "checksum %(checksum_hex)s"),
+        LOG.debug(("Wrote %(bytes_written)d bytes to %(filepath)s with "
+                   "checksum %(checksum_hex)s and multihash %(hash_hex)s"),
                   {'bytes_written': bytes_written,
                    'filepath': filepath,
-                   'checksum_hex': checksum_hex})
+                   'checksum_hex': checksum_hex,
+                   'hash_hex': hash_hex})
 
         if self.backend_group:
             fstore_perm = getattr(
@@ -738,7 +766,11 @@ class Store(glance_store.driver.Store):
         if self.backend_group:
             metadata['backend'] = u"%s" % self.backend_group
 
-        return ('file://%s' % filepath, bytes_written, checksum_hex, metadata)
+        return ('file://%s' % filepath,
+                bytes_written,
+                checksum_hex,
+                hash_hex,
+                metadata)
 
     @staticmethod
     def _delete_partial(filepath, iid):
