@@ -24,6 +24,7 @@ import logging
 import math
 
 from oslo_config import cfg
+from oslo_utils import encodeutils
 from oslo_utils import units
 from six.moves import urllib
 
@@ -42,7 +43,6 @@ except ImportError:
     rbd = None
 
 DEFAULT_POOL = 'images'
-DEFAULT_CONFFILE = '/etc/ceph/ceph.conf'
 DEFAULT_USER = None    # let librados decide based on the Ceph conf file
 DEFAULT_CHUNKSIZE = 8  # in MiB
 DEFAULT_SNAPNAME = 'snap'
@@ -52,7 +52,7 @@ LOG = logging.getLogger(__name__)
 _RBD_OPTS = [
     cfg.IntOpt('rbd_store_chunk_size', default=DEFAULT_CHUNKSIZE,
                min=1,
-               help=_("""
+               help="""
 Size, in megabytes, to chunk RADOS images into.
 
 Provide an integer value representing the size in megabytes to chunk
@@ -70,9 +70,9 @@ Possible Values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.StrOpt('rbd_store_pool', default=DEFAULT_POOL,
-               help=_("""
+               help="""
 RADOS pool in which images are stored.
 
 When RBD is used as the storage backend for storing Glance images, the
@@ -90,9 +90,9 @@ Possible Values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.StrOpt('rbd_store_user', default=DEFAULT_USER,
-               help=_("""
+               help="""
 RADOS user to authenticate as.
 
 This configuration option takes in the RADOS user to authenticate as.
@@ -108,17 +108,24 @@ Possible Values:
 Related options:
     * rbd_store_ceph_conf
 
-""")),
-    cfg.StrOpt('rbd_store_ceph_conf', default=DEFAULT_CONFFILE,
-               help=_("""
+"""),
+    cfg.StrOpt('rbd_store_ceph_conf', default='',
+               help="""
 Ceph configuration file path.
 
-This configuration option takes in the path to the Ceph configuration
+This configuration option specifies the path to the Ceph configuration
 file to be used. If the value for this option is not set by the user
-or is set to None, librados will locate the default configuration file
-which is located at /etc/ceph/ceph.conf. If using Cephx
-authentication, this file should include a reference to the right
-keyring in a client.<USER> section
+or is set to the empty string, librados will read the standard ceph.conf
+file by searching the default Ceph configuration file locations in
+sequential order.  See the Ceph documentation for details.
+
+NOTE: If using Cephx authentication, this file should include a reference
+to the right keyring in a client.<USER> section
+
+NOTE 2: If you leave this option empty (the default), the actual Ceph
+configuration file used may change depending on what version of librados
+is being used.  If it is important for you to know exactly which configuration
+file is in effect, you may specify that file here using this option.
 
 Possible Values:
     * A valid path to a configuration file
@@ -126,9 +133,9 @@ Possible Values:
 Related options:
     * rbd_store_user
 
-""")),
+"""),
     cfg.IntOpt('rados_connect_timeout', default=0,
-               help=_("""
+               help="""
 Timeout value for connecting to Ceph cluster.
 
 This configuration option takes in the timeout value in seconds used
@@ -144,7 +151,7 @@ Possible Values:
 Related options:
     * None
 
-"""))
+"""),
 ]
 
 
@@ -241,7 +248,7 @@ class ImageIterator(object):
                             data = image.read(size - bytes_left, length)
                             bytes_left -= len(data)
                             yield data
-                        raise StopIteration()
+                        return
         except rbd.ImageNotFound:
             raise exceptions.NotFound(
                 _('RBD image %s does not exist') % self.name)
@@ -312,6 +319,23 @@ class Store(driver.Store):
             LOG.error(reason)
             raise exceptions.BadStoreConfiguration(store_name='rbd',
                                                    reason=reason)
+        if self.backend_group:
+            self._set_url_prefix()
+
+    def _set_url_prefix(self):
+        fsid = None
+        with self.get_connection(conffile=self.conf_file,
+                                 rados_id=self.user) as conn:
+            if hasattr(conn, 'get_fsid'):
+                fsid = encodeutils.safe_decode(conn.get_fsid())
+
+        if fsid and self.pool:
+            # ensure nothing contains / or any other url-unsafe character
+            safe_fsid = urllib.parse.quote(fsid, '')
+            safe_pool = urllib.parse.quote(self.pool, '')
+            self._url_prefix = "rbd://%s/%s/" % (safe_fsid, safe_pool)
+        else:
+            self._url_prefix = "rbd://"
 
     @capabilities.check
     def get(self, location, offset=0, chunk_size=None, context=None):
@@ -440,8 +464,9 @@ class Store(driver.Store):
             # Such exception is not dangerous for us so it will be just logged
             LOG.debug("Snapshot %s is unprotected already" % snap_name)
 
+    @driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """
         Stores an image file with supplied identifier to the backend
@@ -451,20 +476,35 @@ class Store(driver.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
+        :param context: A context object
         :param verifier: An object used to verify signatures for images
 
-        :returns: tuple of URL in backing store, bytes written, checksum
-                and a dictionary with storage system specific information
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
         :raises: `glance_store.exceptions.Duplicate` if the image already
-                existed
+                 exists
         """
         checksum = hashlib.md5()
+        os_hash_value = hashlib.new(str(hashing_algo))
         image_name = str(image_id)
         with self.get_connection(conffile=self.conf_file,
                                  rados_id=self.user) as conn:
             fsid = None
             if hasattr(conn, 'get_fsid'):
-                fsid = conn.get_fsid()
+                # Librados's get_fsid is represented as binary
+                # in py3 instead of str as it is in py2.
+                # This is causing problems with ceph.
+                # Decode binary to str fixes these issues.
+                # Fix with encodeutils.safe_decode CAN BE REMOVED
+                # after librados's fix will be stable.
+                #
+                # More informations:
+                # https://bugs.launchpad.net/glance-store/+bug/1816721
+                # https://bugs.launchpad.net/cinder/+bug/1816468
+                # https://tracker.ceph.com/issues/38381
+                fsid = encodeutils.safe_decode(conn.get_fsid())
             with conn.open_ioctx(self.pool) as ioctx:
                 order = int(math.log(self.WRITE_CHUNKSIZE, 2))
                 LOG.debug('creating image %s with order %d and size %d',
@@ -502,12 +542,28 @@ class Store(driver.Store):
                             LOG.debug(_("writing chunk at offset %s") %
                                       (offset))
                             offset += image.write(chunk, offset)
+                            os_hash_value.update(chunk)
                             checksum.update(chunk)
                             if verifier:
                                 verifier.update(chunk)
                         if loc.snapshot:
                             image.create_snap(loc.snapshot)
                             image.protect_snap(loc.snapshot)
+                except rbd.NoSpace:
+                    log_msg = (_LE("Failed to store image %(img_name)s "
+                                   "insufficient space available") %
+                               {'img_name': image_name})
+                    LOG.error(log_msg)
+
+                    # Delete image if one was created
+                    try:
+                        target_pool = loc.pool or self.pool
+                        self._delete_image(target_pool, loc.image,
+                                           loc.snapshot)
+                    except exceptions.NotFound:
+                        pass
+
+                    raise exceptions.StorageFull(message=log_msg)
                 except Exception as exc:
                     log_msg = (_LE("Failed to store image %(img_name)s "
                                    "Store Exception %(store_exc)s") %
@@ -534,7 +590,11 @@ class Store(driver.Store):
         if self.backend_group:
             metadata['backend'] = u"%s" % self.backend_group
 
-        return (loc.get_uri(), image_size, checksum.hexdigest(), metadata)
+        return (loc.get_uri(),
+                image_size,
+                checksum.hexdigest(),
+                os_hash_value.hexdigest(),
+                metadata)
 
     @capabilities.check
     def delete(self, location, context=None):
