@@ -44,7 +44,7 @@ _SHEEPDOG_OPTS = [
     cfg.IntOpt('sheepdog_store_chunk_size',
                min=1,
                default=DEFAULT_CHUNKSIZE,
-               help=_("""
+               help="""
 Chunk size for images to be stored in Sheepdog data store.
 
 Provide an integer value representing the size in mebibyte
@@ -64,10 +64,10 @@ Possible values:
 Related Options:
     * None
 
-""")),
+"""),
     cfg.PortOpt('sheepdog_store_port',
                 default=DEFAULT_PORT,
-                help=_("""
+                help="""
 Port number on which the sheep daemon will listen.
 
 Provide an integer value representing a valid port number on
@@ -86,10 +86,10 @@ Possible values:
 Related Options:
     * sheepdog_store_address
 
-""")),
+"""),
     cfg.HostAddressOpt('sheepdog_store_address',
                        default=DEFAULT_ADDR,
-                       help=_("""
+                       help="""
 Address to bind the Sheepdog daemon to.
 
 Provide a string value representing the address to bind the
@@ -110,7 +110,7 @@ Possible values:
 Related Options:
     * sheepdog_store_port
 
-"""))
+"""),
 ]
 
 
@@ -256,7 +256,7 @@ class ImageIterator(object):
             data = image.read(total - left, length)
             left -= len(data)
             yield data
-        raise StopIteration()
+        return
 
 
 class Store(glance_store.driver.Store):
@@ -270,6 +270,10 @@ class Store(glance_store.driver.Store):
     def get_schemes(self):
         return ('sheepdog',)
 
+    def _set_url_prefix(self):
+        self._url_prefix = "%s://%s:%s:" % (
+            'sheepdog', self.addr, self.port)
+
     def configure_add(self):
         """
         Configure the Store to use the stored configuration options
@@ -304,6 +308,9 @@ class Store(glance_store.driver.Store):
             raise exceptions.BadStoreConfiguration(store_name='sheepdog',
                                                    reason=reason)
 
+        if self.backend_group:
+            self._set_url_prefix()
+
     @capabilities.check
     def get(self, location, offset=0, chunk_size=None, context=None):
         """
@@ -343,8 +350,9 @@ class Store(glance_store.driver.Store):
                                       % image.name)
         return image.get_size()
 
+    @glance_store.driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """
         Stores an image file with supplied identifier to the backend
@@ -354,11 +362,15 @@ class Store(glance_store.driver.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
+        :param context: A context object
         :param verifier: An object used to verify signatures for images
 
-        :returns: tuple of URL in backing store, bytes written, and checksum
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
         :raises: `glance_store.exceptions.Duplicate` if the image already
-                existed
+                 exists
         """
 
         image = SheepdogImage(self.addr, self.port, image_id,
@@ -377,6 +389,7 @@ class Store(glance_store.driver.Store):
 
         try:
             offset = 0
+            os_hash_value = hashlib.new(str(hashing_algo))
             checksum = hashlib.md5()
             chunks = utils.chunkreadable(image_file, self.WRITE_CHUNKSIZE)
             for chunk in chunks:
@@ -389,6 +402,7 @@ class Store(glance_store.driver.Store):
                     image.resize(offset + chunk_length)
                 image.write(chunk, offset, chunk_length)
                 offset += chunk_length
+                os_hash_value.update(chunk)
                 checksum.update(chunk)
                 if verifier:
                     verifier.update(chunk)
@@ -402,7 +416,11 @@ class Store(glance_store.driver.Store):
         if self.backend_group:
             metadata['backend'] = u"%s" % self.backend_group
 
-        return (location.get_uri(), offset, checksum.hexdigest(), metadata)
+        return (location.get_uri(),
+                offset,
+                checksum.hexdigest(),
+                os_hash_value.hexdigest(),
+                metadata)
 
     @capabilities.check
     def delete(self, location, context=None):
