@@ -61,7 +61,7 @@ STORE_SCHEME = 'vsphere'
 _VMWARE_OPTS = [
     cfg.HostAddressOpt('vmware_server_host',
                        sample_default='127.0.0.1',
-                       help=_("""
+                       help="""
 Address of the ESX/ESXi or vCenter Server target system.
 
 This configuration option sets the address of the ESX/ESXi or vCenter
@@ -77,10 +77,10 @@ Related options:
     * vmware_server_username
     * vmware_server_password
 
-""")),
+"""),
     cfg.StrOpt('vmware_server_username',
                sample_default='root',
-               help=_("""
+               help="""
 Server username.
 
 This configuration option takes the username for authenticating with
@@ -95,10 +95,10 @@ Related options:
     * vmware_server_host
     * vmware_server_password
 
-""")),
+"""),
     cfg.StrOpt('vmware_server_password',
                sample_default='vmware',
-               help=_("""
+               help="""
 Server password.
 
 This configuration option takes the password for authenticating with
@@ -113,12 +113,12 @@ Related options:
     * vmware_server_host
     * vmware_server_username
 
-"""),
+""",
                secret=True),
     cfg.IntOpt('vmware_api_retry_count',
                default=10,
                min=1,
-               help=_("""
+               help="""
 The number of VMware API retries.
 
 This configuration option specifies the number of times the VMware
@@ -132,11 +132,11 @@ Possible Values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.IntOpt('vmware_task_poll_interval',
                default=5,
                min=1,
-               help=_("""
+               help="""
 Interval in seconds used for polling remote tasks invoked on VMware
 ESX/VC server.
 
@@ -149,10 +149,10 @@ Possible Values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.StrOpt('vmware_store_image_dir',
                default=DEFAULT_STORE_IMAGE_DIR,
-               help=_("""
+               help="""
 The directory where the glance images will be stored in the datastore.
 
 This configuration option specifies the path to the directory where the
@@ -166,11 +166,11 @@ Possible Values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.BoolOpt('vmware_insecure',
                 default=False,
                 deprecated_name='vmware_api_insecure',
-                help=_("""
+                help="""
 Set verification of the ESX/vCenter server certificate.
 
 This configuration option takes a boolean value to determine
@@ -190,10 +190,10 @@ Possible Values:
 Related options:
     * vmware_ca_file
 
-""")),
+"""),
     cfg.StrOpt('vmware_ca_file',
                sample_default='/etc/ssl/certs/ca-certificates.crt',
-               help=_("""
+               help="""
 Absolute path to the CA bundle file.
 
 This configuration option enables the operator to use a custom
@@ -209,10 +209,10 @@ Possible Values:
 Related options:
     * vmware_insecure
 
-""")),
+"""),
     cfg.MultiStrOpt(
         'vmware_datastores',
-        help=_("""
+        help="""
 The datastores where the image can be stored.
 
 This configuration option specifies the datastores where the image can
@@ -237,7 +237,7 @@ Possible Values:
 Related options:
    * None
 
-"""))]
+""")]
 
 
 def http_response_iterator(conn, response, size):
@@ -258,16 +258,18 @@ def http_response_iterator(conn, response, size):
 
 class _Reader(object):
 
-    def __init__(self, data, verifier=None):
+    def __init__(self, data, hashing_algo, verifier=None):
         self._size = 0
         self.data = data
         self.checksum = hashlib.md5()
+        self.os_hash_value = hashlib.new(str(hashing_algo))
         self.verifier = verifier
 
     def read(self, size=None):
         result = self.data.read(size)
         self._size += len(result)
         self.checksum.update(result)
+        self.os_hash_value.update(result)
         if self.verifier:
             self.verifier.update(result)
         return result
@@ -461,8 +463,9 @@ class Store(glance_store.Store):
             raise exceptions.BadStoreConfiguration(
                 store_name='vmware_datastore', reason=msg)
         if len(parts) == 3 and parts[2]:
-            weight = parts[2]
-            if not weight.isdigit():
+            try:
+                weight = int(parts[2])
+            except ValueError:
                 msg = (_('Invalid weight value %(weight)s in '
                          'vmware_datastores configuration') %
                        {'weight': weight})
@@ -499,7 +502,7 @@ class Store(glance_store.Store):
                 LOG.error(msg)
                 raise exceptions.BadStoreConfiguration(
                     store_name='vmware_datastore', reason=msg)
-            ds_map.setdefault(int(weight), []).append(ds_obj)
+            ds_map.setdefault(weight, []).append(ds_obj)
         return ds_map
 
     def configure_add(self):
@@ -512,6 +515,19 @@ class Store(glance_store.Store):
             store_conf = self.conf.glance_store
 
         self.store_image_dir = store_conf.vmware_store_image_dir
+        if self.backend_group:
+            self._set_url_prefix()
+
+    def _set_url_prefix(self):
+        path = os.path.join(DS_URL_PREFIX,
+                            self.store_image_dir)
+
+        if netutils.is_valid_ipv6(self.server_host):
+            self._url_prefix = '%s://[%s]%s' % (self.scheme,
+                                                self.server_host, path)
+        else:
+            self._url_prefix = '%s://%s%s' % (self.scheme,
+                                              self.server_host, path)
 
     def select_datastore(self, image_size):
         """Select a datastore with free space larger than image size."""
@@ -554,8 +570,9 @@ class Store(glance_store.Store):
             cookie = list(vim_cookies)[0]
             return cookie.name + '=' + cookie.value
 
+    @glance_store.driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """Stores an image file with supplied identifier to the backend
         storage system and returns a tuple containing information
@@ -564,17 +581,21 @@ class Store(glance_store.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
+        :param context: A context object
         :param verifier: An object used to verify signatures for images
-        :returns: tuple of URL in backing store, bytes written, checksum
-                and a dictionary with storage system specific information
-        :raises: `glance.common.exceptions.Duplicate` if the image already
-                existed
-                `glance.common.exceptions.UnexpectedStatus` if the upload
-                request returned an unexpected status. The expected responses
-                are 201 Created and 200 OK.
+
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
+        :raises: `glance_store.exceptions.Duplicate` if the image already
+                 exists
+        :raises: `glance.common.exceptions.UnexpectedStatus` if the upload
+                 request returned an unexpected status. The expected responses
+                 are 201 Created and 200 OK.
         """
         ds = self.select_datastore(image_size)
-        image_file = _Reader(image_file, verifier)
+        image_file = _Reader(image_file, hashing_algo, verifier)
         headers = {}
         if image_size > 0:
             headers.update({'Content-Length': six.text_type(image_size)})
@@ -638,8 +659,11 @@ class Store(glance_store.Store):
         if self.backend_group:
             metadata['backend'] = u"%s" % self.backend_group
 
-        return (loc.get_uri(), image_file.size,
-                image_file.checksum.hexdigest(), metadata)
+        return (loc.get_uri(),
+                image_file.size,
+                image_file.checksum.hexdigest(),
+                image_file.os_hash_value.hexdigest(),
+                metadata)
 
     @capabilities.check
     def get(self, location, offset=0, chunk_size=None, context=None):
